----------------------------------------------------------------------------------------------------
-- Company: 		Trenz Electronic
-- Engineer: 		Oleksandr Kiyenko
----------------------------------------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.all;
----------------------------------------------------------------------------------------------------
entity vga_ctrl IS
generic(
	C_COLOR_BITS	: integer	:= 4;
	C_MODE			: integer	:= 0
);
port(
	pixel_clk		: in  STD_LOGIC;	--pixel clock at frequency of VGA mode being used
	reset_n			: in  STD_LOGIC;	--active low asycnchronous reset
	h_sync			: out STD_LOGIC;	--horiztonal sync pulse
	v_sync			: out STD_LOGIC;	--vertical sync pulse
	red_out			: out STD_LOGIC_VECTOR(C_COLOR_BITS-1 downto 0);
	green_out		: out STD_LOGIC_VECTOR(C_COLOR_BITS-1 downto 0);
	blue_out		: out STD_LOGIC_VECTOR(C_COLOR_BITS-1 downto 0)
);
end vga_ctrl;
----------------------------------------------------------------------------------------------------
architecture behavior OF vga_ctrl IS
----------------------------------------------------------------------------------------------------
component vga_controller IS
generic(
	h_pulse 		: integer	:= 208;		--horiztonal sync pulse width in pixels
	h_bp	 		: integer	:= 336;		--horiztonal back porch width in pixels
	h_pixels		: integer	:= 1920;	--horiztonal display width in pixels
	h_fp	 		: integer	:= 128;		--horiztonal front porch width in pixels
	h_pol			: STD_LOGIC	:= '0';		--horizontal sync pulse polarity (1 = positive, 0 = negative)
	v_pulse 		: integer	:= 3;		--vertical sync pulse width in rows
	v_bp	 		: integer	:= 38;		--vertical back porch width in rows
	v_pixels		: integer	:= 1200;	--vertical display width in rows
	v_fp	 		: integer	:= 1;		--vertical front porch width in rows
	v_pol			: STD_LOGIC	:= '1'		--vertical sync pulse polarity (1 = positive, 0 = negative)
);
port(
	pixel_clk		: in  STD_LOGIC;	--pixel clock at frequency of VGA mode being used
	reset_n			: in  STD_LOGIC;	--active low asycnchronous reset
	h_sync			: out STD_LOGIC;	--horiztonal sync pulse
	v_sync			: out STD_LOGIC;	--vertical sync pulse
	disp_ena		: out STD_LOGIC;	--display enable ('1' = display time, '0' = blanking time)
	column			: out integer;		--horizontal pixel coordinate
	row				: out integer;		--vertical pixel coordinate
	n_blank			: out STD_LOGIC;	--direct blacking output to DAC
	n_sync			: out STD_LOGIC		--sync-on-green output to DAC
);
end component;

component hw_image_generator IS
generic(
	pixels_y 		: integer := 478;   --row that first color will persist until
	pixels_x 		: integer := 600	--column that first color will persist until
);	
port(
	disp_ena 		: in   STD_LOGIC;	--display enable ('1' = display time, '0' = blanking time)
	row      		: in   integer;		--row pixel coordinate
	column   		: in   integer;		--column pixel coordinate
	red      		: out  STD_LOGIC_VECTOR(7 DOWNTO 0) := (OTHERS => '0'); --red magnitude output to DAC
	green    		: out  STD_LOGIC_VECTOR(7 DOWNTO 0) := (OTHERS => '0'); --green magnitude output to DAC
	blue     		: out  STD_LOGIC_VECTOR(7 DOWNTO 0) := (OTHERS => '0')	--blue magnitude output to DAC
);	
end component;

-- https://www.digikey.com/eewiki/pages/viewpage.action?pageId=15925278

-- -- 640x480@60 
-- constant H_PULSE 	: integer := 96;    	--horiztonal sync pulse width in pixels
-- constant H_BP	 	: integer := 48;		--horiztonal back porch width in pixels
-- constant H_PIXELS	: integer := 640;		--horiztonal display width in pixels
-- constant H_FP	 	: integer := 16;		--horiztonal front porch width in pixels
-- constant V_PULSE 	: integer := 2;			--vertical sync pulse width in rows
-- constant V_BP	 	: integer := 33;		--vertical back porch width in rows
-- constant V_PIXELS	: integer := 480;		--vertical display width in rows
-- constant V_FP	 	: integer := 10;		--vertical front porch width in rows
-- constant PIXELS_Y 	: integer := 320;   	--row that first color will persist until
-- constant PIXELS_X 	: integer := 240;  		--column that first color will persist until
-- constant H_POL		: STD_LOGIC := '0';		--horizontal sync pulse polarity (1 = positive, 0 = negative)
-- constant V_POL		: STD_LOGIC := '0';		--vertical sync pulse polarity (1 = positive, 0 = negative)

-- 1024x768@60 
constant H_PULSE 	: integer := 136;    	--horiztonal sync pulse width in pixels
constant H_BP	 	: integer := 160;		--horiztonal back porch width in pixels
constant H_PIXELS	: integer := 1024;		--horiztonal display width in pixels
constant H_FP	 	: integer := 24;		--horiztonal front porch width in pixels
constant V_PULSE 	: integer := 6;			--vertical sync pulse width in rows
constant V_BP	 	: integer := 29;		--vertical back porch width in rows
constant V_PIXELS	: integer := 768;		--vertical display width in rows
constant V_FP	 	: integer := 3;			--vertical front porch width in rows
constant PIXELS_Y 	: integer := 320;   	--row that first color will persist until
constant PIXELS_X 	: integer := 240;  		--column that first color will persist until
constant H_POL		: STD_LOGIC := '0';		--horizontal sync pulse polarity (1 = positive, 0 = negative)
constant V_POL		: STD_LOGIC := '0';		--vertical sync pulse polarity (1 = positive, 0 = negative)

signal column		: integer;
signal row			: integer;
signal disp_ena		: STD_LOGIC;
signal red			: STD_LOGIC_VECTOR(7 downto 0);
signal green		: STD_LOGIC_VECTOR(7 downto 0);
signal blue			: STD_LOGIC_VECTOR(7 downto 0);
signal n_blank		: STD_LOGIC;
signal n_sync		: STD_LOGIC;
----------------------------------------------------------------------------------------------------
begin
----------------------------------------------------------------------------------------------------
ctrl_inst: vga_controller
generic map(
	h_pulse 	=> H_PULSE,    	--horiztonal sync pulse width in pixels
	h_bp	 	=> H_BP,		--horiztonal back porch width in pixels
	h_pixels	=> H_PIXELS,	--horiztonal display width in pixels
	h_fp	 	=> H_FP,		--horiztonal front porch width in pixels
	h_pol		=> H_POL,		--horizontal sync pulse polarity (1 = positive, 0 = negative)
	v_pulse 	=> V_PULSE,		--vertical sync pulse width in rows
	v_bp	 	=> V_BP,		--vertical back porch width in rows
	v_pixels	=> V_PIXELS,	--vertical display width in rows
	v_fp	 	=> V_FP,		--vertical front porch width in rows
	v_pol		=> V_POL		--vertical sync pulse polarity (1 = positive, 0 = negative)
)
port map(
	pixel_clk	=> pixel_clk,	--pixel clock at frequency of VGA mode being used
	reset_n		=> reset_n,		--active low asycnchronous reset
	h_sync		=> h_sync,		--horiztonal sync pulse
	v_sync		=> v_sync,		--vertical sync pulse
	disp_ena	=> disp_ena,	--display enable ('1' = display time, '0' = blanking time)
	column		=> column,		--horizontal pixel coordinate
	row			=> row,			--vertical pixel coordinate
	n_blank		=> n_blank,		--direct blacking output to DAC
	n_sync		=> n_sync		--sync-on-green output to DAC
);

gen_inst: hw_image_generator
generic map(
	pixels_y 	=> PIXELS_Y,	--row that first color will persist until
	pixels_x	=> PIXELS_X		--column that first color will persist until
)
port map(
	disp_ena 	=> disp_ena,	--display enable ('1' = display time, '0' = blanking time)
	row      	=> row,			--row pixel coordinate
	column   	=> column,		--column pixel coordinate
	red      	=> red,			--red magnitude output to DAC
	green    	=> green,		--green magnitude output to DAC
	blue     	=> blue			--blue magnitude output to DAC
);

red_out			<= red(7 downto 8-C_COLOR_BITS);
green_out		<= green(7 downto 8-C_COLOR_BITS);
blue_out		<= blue(7 downto 8-C_COLOR_BITS);
----------------------------------------------------------------------------------------------------
end behavior;